Index: defs.h
===================================================================
RCS file: /cvsroot/xen_ia64/people/xencrash/src/crash/defs.h,v
retrieving revision 1.9
retrieving revision 1.9.4.1
diff -u -r1.9 -r1.9.4.1
--- defs.h	28 Apr 2007 02:19:07 -0000	1.9
+++ defs.h	7 May 2007 07:46:15 -0000	1.9.4.1
@@ -3941,6 +3941,8 @@
 int kdump_memory_dump(FILE *);
 void get_kdump_regs(struct bt_info *, ulong *, ulong *);
 void xen_kdump_p2m_mfn(char *);
+void check_netdump_xen(void);
+void check_kdump_xen(void);
 
 /*
  *  diskdump.c
Index: ia64.c
===================================================================
RCS file: /cvsroot/xen_ia64/people/xencrash/src/crash/ia64.c,v
retrieving revision 1.5
retrieving revision 1.5.4.1
diff -u -r1.5 -r1.5.4.1
--- ia64.c	28 Apr 2007 02:19:07 -0000	1.5
+++ ia64.c	7 May 2007 07:46:15 -0000	1.5.4.1
@@ -3856,6 +3856,85 @@
  *  up the dom0 kernel's complete phys_to_machine_mapping[max_pfn] array.
  */
 
+#define MAX_IA64_FRAMES PTRS_PER_PGD
+#define MFNS_PER_FRAME PTRS_PER_PMD
+
+static int
+ia64_xen_kdump_p2m_create(struct xen_kdump_data *xkd)
+{
+	int i, j;
+	ulong *up;
+	ulong frames;
+	ulong *frame_mfn;
+	int last_idx = 0;
+	int *mfns;
+
+	/*
+	 *  Temporarily read physical (machine) addresses from vmcore by
+	 *  going directly to read_netdump() instead of via read_kdump().
+	 */
+	pc->readmem = read_netdump;
+
+	if (CRASHDEBUG(1))
+		fprintf(fp, "ia64_xen_kdump_p2m_create: p2m_mfn: %lx\n", xkd->p2m_mfn);
+
+	if (!readmem(PTOB(xkd->p2m_mfn), PHYSADDR, xkd->page, PAGESIZE(), 
+	    "xen kdump p2m mfn page", RETURN_ON_ERROR))
+		error(FATAL, "cannot read xen kdump p2m mfn page\n");
+
+	if (CRASHDEBUG(2))
+		ia64_debug_dump_page(fp, xkd->page, "domain pgd");
+
+	frame_mfn = (ulong *)calloc(MAX_IA64_FRAMES * sizeof(ulong), 1);
+	mfns = (int *)calloc(MAX_IA64_FRAMES * sizeof(int), 1);
+	if (!frame_mfn || !mfns)
+		error(FATAL, "cannot malloc xen kdump p2m pages\n");
+
+	for (i = 0, up = (ulong *)xkd->page; i < MAX_IA64_FRAMES; i++, up++) {
+		frame_mfn[i] = (*up & _PFN_MASK) >> 14; /* XXX 14 */
+		if (frame_mfn[i])
+			last_idx = i;
+	}
+
+	xkd->p2m_frames = (last_idx + 1) * MFNS_PER_FRAME;
+
+	if (CRASHDEBUG(1))
+		fprintf(fp, "p2m_frames: %d\n", xkd->p2m_frames);
+
+	if ((xkd->p2m_mfn_frame_list = (ulong *)
+	    malloc(xkd->p2m_frames * sizeof(ulong))) == NULL)
+		error(FATAL, "cannot malloc p2m_frame_index_list");
+
+	for (i = 0; i <= last_idx; i++) {
+		if (!frame_mfn[i])
+			continue;
+
+		if (!readmem(PTOB(frame_mfn[i]), PHYSADDR, xkd->page,
+		    PAGESIZE(), "xen kdump p2m mfn list page", RETURN_ON_ERROR))
+			error(FATAL, "cannot read xen kdump p2m mfn list page\n");
+
+		for (j = 0, up = (ulong *)xkd->page; j < MFNS_PER_FRAME; j++, up++)
+			if (*up & _PFN_MASK)
+				xkd->p2m_mfn_frame_list[i *  MFNS_PER_FRAME + j] =
+				    (*up & _PFN_MASK) >> 14;
+
+		if (CRASHDEBUG(7))
+			ia64_debug_dump_page(fp, xkd->page, "pfn_to_mfn_frame_list page");
+	}
+
+
+	if (CRASHDEBUG(2)) {
+		for (i = 0; i < xkd->p2m_frames; i++)
+			fprintf(fp, "%lx ", xkd->p2m_mfn_frame_list[i]);
+		fprintf(fp, "\n");
+	}
+
+	pc->readmem = read_kdump;
+
+	return TRUE;
+}
+
+#if 0
 static int
 ia64_xen_kdump_p2m_create(struct xen_kdump_data *xkd)
 {
@@ -3868,6 +3947,7 @@
 
 	return FALSE;
 }
+#endif
 
 static char *
 ia64_xen_kdump_load_page(ulong kvaddr, char *pgbuf)
Index: main.c
===================================================================
RCS file: /cvsroot/xen_ia64/people/xencrash/src/crash/main.c,v
retrieving revision 1.7
retrieving revision 1.7.6.1
diff -u -r1.7 -r1.7.6.1
--- main.c	20 Apr 2007 04:40:21 -0000	1.7
+++ main.c	7 May 2007 07:46:15 -0000	1.7.6.1
@@ -305,6 +305,8 @@
                                 pc->readmem = read_netdump;
                                 pc->writemem = write_netdump;
 
+				check_netdump_xen();
+
                         } else if (is_kdump(argv[optind], KDUMP_LOCAL)) {
                                 if (pc->flags & MEMORY_SOURCES) {
                                         error(INFO,
@@ -315,6 +317,8 @@
                                 pc->dumpfile = argv[optind];
                                 pc->readmem = read_kdump;
                                 pc->writemem = write_kdump;
+
+				check_kdump_xen();
 
                         } else if (is_xendump(argv[optind])) {
                                 if (pc->flags & MEMORY_SOURCES) {
Index: netdump.c
===================================================================
RCS file: /cvsroot/xen_ia64/people/xencrash/src/crash/netdump.c,v
retrieving revision 1.5
retrieving revision 1.5.6.1
diff -u -r1.5 -r1.5.6.1
--- netdump.c	20 Apr 2007 04:40:21 -0000	1.5
+++ netdump.c	7 May 2007 07:46:15 -0000	1.5.6.1
@@ -2096,8 +2096,14 @@
 	pfn = (ulong)BTOP(pseudo);
 	mfn_idx = pfn / (PAGESIZE()/sizeof(ulong));
 	frame_idx = pfn % (PAGESIZE()/sizeof(ulong));
+	if (mfn_idx >= xkd->p2m_frames)
+		return P2M_FAILURE;
 	mfn_frame = xkd->p2m_mfn_frame_list[mfn_idx];
 
+#ifdef IA64
+	if (mfn_frame == 0)
+		return P2M_FAILURE;
+#endif
 	if (mfn_frame == xkd->last_mfn_read)
 		xkd->cache_hits++;
 	else if (read_netdump(0, xkd->page, PAGESIZE(), 0, 
@@ -2107,7 +2113,13 @@
 	xkd->last_mfn_read = mfn_frame;
 
 	mfnptr = ((ulong *)(xkd->page)) + frame_idx;
+#ifdef IA64
+	if (!(*mfnptr & 0x1))
+		return P2M_FAILURE;
+	paddr = *mfnptr & _PFN_MASK;
+#else
 	paddr = (physaddr_t)PTOB((ulonglong)(*mfnptr));  
+#endif
 	paddr |= PAGEOFFSET(pseudo);
 
 	if (CRASHDEBUG(7))
@@ -2149,4 +2161,29 @@
 				value); 
 	} else 
 		error(WARNING, "invalid p2m_mfn argument: %s\n", arg);
+}
+
+void
+check_netdump_xen(void)
+{
+	if (xen_kdump_data.p2m_mfn) {
+		if (!XEN_CORE_DUMPFILE()) {
+			pc->flags |= XEN_CORE;
+			nd->xen_kdump_data = &xen_kdump_data;
+			nd->xen_kdump_data->last_mfn_read = BADVAL;
+			nd->xen_kdump_data->flags |= KDUMP_MFN_LIST;
+		}
+              	pc->readmem = read_kdump;
+	}
+}
+
+void
+check_kdump_xen(void)
+{
+	if (!XEN_CORE_DUMPFILE() && xen_kdump_data.p2m_mfn) {
+		pc->flags |= XEN_CORE;
+		nd->xen_kdump_data = &xen_kdump_data;
+		nd->xen_kdump_data->last_mfn_read = BADVAL;
+		nd->xen_kdump_data->flags |= KDUMP_MFN_LIST;
+	}
 }
